/*jslint plusplus: true */

// Data for each payoff matrix will be stored here as an "associative array", e.g.
// payoffMatrix["U526-2012-2013-min10"] though technically the data will be stored
// as array object members since that is how JavaScript works.
var payoffMatrix = [];

function isOldIE() {
  "use strict";
  // Not perfect, but good enough.
  var oldIE = 0;
  if (navigator.appName.indexOf("Internet Explorer") !== -1) {
    oldIE = navigator.appVersion.indexOf("MSIE 9") === -1 &&
            navigator.appVersion.indexOf("MSIE 1") === -1;
  }
  return oldIE;
}

function pm_addPairNames(img, id, clx, cly) {
  // IMG - Image object (use this as argument in onload call)
  // ID  - String index into payoffMatrix[]
  // CLX - Class name for each <span> element created for X-axis pair names
  // CLY - Class name for each <span> element created for Y-axis pair names             

  "use strict";
    
  var names1, names2, px, xoff, yoff, div, spn, pg, i, pairname;

  names1 = payoffMatrix[id].pairs.fullname1;
  names2 = payoffMatrix[id].pairs.fullname2;
  
  // Number of pixels per pair
  px = img.width / names1.length;
  
  // Pair names will be absolutely positioned relative to the image.
  xoff = img.offsetLeft;
  yoff = img.offsetTop;

  div = img.parentElement;

  // position needs to be 'absolute' but this is imposed across the board by
  // the CSS for the CLX and CLY classes. For y-axis labels, CLY class also
  // imposed left: 0 positioning.
  for (i=0; i<names1.length; i++) {
    // Convert spaces to non-breaking spaces to try to avoid line wrapping if
    // pair names run out of room (if say the font size is bumped up).
    pairname = (names1[i] + ' - ' + names2[i]).replace(/ /g, '&nbsp;');

    // y-axis pair name
    spn = document.createElement('span');
    spn.innerHTML = pairname;
    spn.style.top = (yoff + i * px) + 'px';
    if (cly !== undefined) { spn.className = cly; }
    div.appendChild(spn);
  }
  
  if ( isOldIE() ) {
    pg = document.createElement('p');
    pg.innerHTML = 'Vertical text of each pair name would appear here if you ' +
    'were viewing this page with Firefox, Chrome, Safari, or Internet Explorer ' +
    '9 or later.';
    pg.style.position = 'absolute';
    pg.style.left = xoff + 'px';
    pg.style.top = (yoff - 100) + 'px';
    pg.style.width = '500px';
    div.appendChild(pg);
    return;
  }
   
  for (i=0; i<names1.length; i++) {
    pairname = (names1[i] + ' - ' + names2[i]).replace(/ /g, '&nbsp;');

    // x-axis pair name
    spn = document.createElement('span');
    spn.innerHTML = pairname;
    spn.style.top = (yoff - 8) + 'px';
    spn.style.left = (xoff + i * px) + 'px';
    if (clx !== undefined) { spn.className = clx; }
    div.appendChild(spn);
  }
}

// Don't really need the set of parentheses that starts before function keyword
// but it keeps jslint happy.
var pm_tooltip = ( function() {
  "use strict";
  
  var tt, spn, h, px, xpos, ypos, ix, iy, ix_last, iy_last, idx_max,
      pr, pm, cnt, pct, meanerr, ou, str1, str2,
      top = 3, left = 3, maxw = 300,
      speed = 10, timer = 20, endalpha = 95, alpha = 0,
      ie = document.all ? true : false;
  
  return {
    show:function(img, id) {
      // IMG  - Image object that mouse is over
      // ID   - String index into payoffMatrix[] for image
      if (tt === undefined) {
        tt = document.createElement('div');
        tt.setAttribute('id', 'pm-tooltip');
        document.body.appendChild(tt);
        tt.style.opacity = 0;
        tt.style.filter = 'alpha(opacity=0)';
        spn = document.createElement('span');
        tt.appendChild(spn);
      }

      // Recalculate the number of pixels per pair (PX) each time the mouse
      // enters an image since PX can vary for different images.
      px = img.width / payoffMatrix[id].pairs.fullname1.length;
      idx_max = payoffMatrix[id].pairs.fullname1.length - 1;

      // Force update of tooltip content on first move. 
      ix_last = -1; iy_last = -1;
      
      tt.style.display = 'block';

      clearInterval(tt.timer);
      tt.timer = setInterval(function(){pm_tooltip.fade(1);}, timer);
    },
    
    move:function(e, img, id, mode, bw) {
      // e    - event
      // img  - image object
      // id   - dataset id
      // mode - 'pp' or 'ou'
      // bw   - border width around image (in pixels)
    
      // Move tooltip, keeping lower left corner at the mouse pointer.
      var u = ie ? event.clientY + document.documentElement.scrollTop  : e.pageY,
          l = ie ? event.clientX + document.documentElement.scrollLeft : e.pageX;
      
      xpos = l - img.offsetLeft -img.offsetParent.offsetLeft;
      ypos = u - img.offsetTop - img.offsetParent.offsetTop;

      ix = Math.floor((xpos-bw) / px);
      iy = Math.floor((ypos-bw) / px);
      
      // If cursor is on image border, act like it is just inside image border.
      ix = Math.min(Math.max(0,ix),idx_max);
      iy = Math.min(Math.max(0,iy),idx_max);
      
      if (ix === ix_last && iy === iy_last) {
        tt.style.top = (u - tt.offsetHeight) + 'px';
        tt.style.left = (l + left) + 'px';
        return;
      }
      
      // Conveniences
      pr = payoffMatrix[id].pairs;
      pm = payoffMatrix[id].pm;
      
      if ( pm[iy] && pm[iy][ix] ) {
        cnt = pm[iy][ix][0]; pct = pm[iy][ix][1];
        meanerr = pm[iy][ix][2]; ou = pm[iy][ix][3]; 
      }
      else if ( pm[ix] && pm[ix][iy] ) {
        cnt = pm[ix][iy][0]; pct = 100 - pm[ix][iy][1];
        meanerr = pm[ix][iy][2]; ou = -pm[ix][iy][3];
      }
      else {
        cnt = 0;
      }

      str1 = pr.pct[iy].toFixed(2) + '%  ' + pr.fullname1[iy] + ' - ' + pr.fullname2[iy] + 
        '<br>' + pr.pct[ix].toFixed(2) + '%  ' +  pr.fullname1[ix] + ' - ' + pr.fullname2[ix];
      if (cnt) {
        str2 = cnt + ' Boards, ';
        if (mode === 'pp') {
          str2 += '<strong>' + pct.toFixed(2) + ' &plusmn; ' +
                   meanerr.toFixed(2) + '%' + '</strong>' + ', OU: ' + ou.toFixed(1) + '%';
        }
        else {
          str2 += '<strong>' + ou.toFixed(1) + '%' + '</strong>' +
          (ou > 0 ? ' (overperform)' : ou < 0 ? ' (underperform)' : '(expected)') +
          ', ' + pct.toFixed(2) + '%';
        }
      }
      else {
        str2 = 'No boards played against each other';
      }
               
      spn.innerHTML = str1 + '<br>' + str2;

      tt.style.top = (u - tt.offsetHeight) + 'px';
      tt.style.left = (l + left) + 'px';
    },

    fade:function(d) {
      var a = alpha, i = speed;
      if ((a !== endalpha && d === 1) || (a !== 0 && d === -1)) {
        if (endalpha - a < speed && d === 1) { i = endalpha - a; }
        else if (alpha < speed && d === -1) { i = a; }
        alpha = a + (i * d);
        tt.style.opacity = alpha * 0.01;
        tt.style.filter = 'alpha(opacity=' + alpha + ')';
      }
      else {
        clearInterval(tt.timer);
        if (d === -1) { tt.style.display = 'none'; }
      }
    },

    hide:function() {
      clearInterval(tt.timer);
      tt.timer = setInterval(function(){pm_tooltip.fade(-1);}, timer);
    }
  };
}() );


